<?php

namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class iFoodMenu
{
    private $iFoodMenuService;

    public function __construct(\App\Service\iFoodMenu $iFoodMenuService)
    {
        $this->iFoodMenuService = $iFoodMenuService;
    }
    public function getMenuId(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();
            $merchantId = isset($params["merchantId"]) ? strval($params["merchantId"]) : "";

            $token = $this->iFoodMenuService->authentication();
            $menu  = $this->iFoodMenuService->getMenuId($merchantId, $token);
            $info  = $this->iFoodMenuService->getMerchantInfo($merchantId, $token);
            $data  = [
                "menu" => $menu,
                "info" => $info,
            ];
            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $data,
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

    public function getMerchantInfo(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();
            $merchantId = isset($params["merchantId"]) ? strval($params["merchantId"]): "";

            $token = $this->iFoodMenuService->authentication();
            $info = $this->iFoodMenuService->getMerchantInfo($merchantId, $token);

            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $info,
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

    public function getCategory(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();
            $menuId = isset($params["menuID"]) ? strval($params["menuID"]) : "";
            $merchantId = isset($params["merchantId"])
                ? strval($params["merchantId"])
                : "";

            $token = $this->iFoodMenuService->authentication();
            $menu = $this->iFoodMenuService->getMenu($menuId, $merchantId, $token );
            $category = $this->iFoodMenuService->GetCategory($menu);

            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $category,
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }
    public function getItens(Request $request, Response $response)
    {
        try {
            $params   = $request->getParsedBody();
            $items = $this->iFoodMenuService->items($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $items,
            ]));

            return $response->withHeader('Content-Type', 'application/json');

        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

    public function getBranch(Request $request, Response $response)
    {
        try {
            $result = $this->iFoodMenuService->getBranch();


            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $result,
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

    public function getBranchIdToken(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();
            $CDFILIAL = $params["cdFilial"];

            $result = $this->iFoodMenuService->getBranchIdToken($CDFILIAL);

            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $result,
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }
    }

    public function setOptionExternalCode(Request $request, Response $response){
        try {
            $params = $request->getParsedBody();
            $token = $this->iFoodMenuService->authentication();
            $cdProduto = '';

            if(!empty($params['cdgrpocor'] && !empty($params['cdocorr']))) {
                $params['cdProduto'] = $this->iFoodMenuService->checkOcurrenceExists($params);
            } else {
                $params['cdProduto'] = $this->iFoodMenuService->checkExternalCodeExists($params);
            }

            $this->iFoodMenuService->logItemAlteration($params);
            $result = $this->iFoodMenuService->setOptionExternalCode($token, $params);
            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $result,
                    "pagination" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }


        return $result;
    }

    public function getAlterationLogs(Request $request, Response $response) {
        try {
            $params = $request->getQueryParams();
            $result = $this->iFoodMenuService->getAlterationLogs($params);
            if (empty($result)) {
                $response->getBody()->write(
                    json_encode([
                        "error" => true,
                        "message" => "Nenhum item foi encontrado.",
                        "data" => [],
                        "pagination" => [
                            "limit" => $params["limit"] ?? 10,
                            "page" => $params["page"] ?? 1,
                        ],
                    ])
                );
            } else {
            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $result[0],
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                        "total" => $result[1],
                    ],
                ])
            );
        }

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }

        return $result;

    }

    public function setItemExternalCode(Request $request, Response $response){
        try {
            $params = $request->getParsedBody();
            $token = $this->iFoodMenuService->authentication();
            $params['cdProduto'] = $this->iFoodMenuService->checkExternalCodeExists($params);
            $this->iFoodMenuService->logItemAlteration($params);
            $result = $this->iFoodMenuService->setItemExternalcode($token, $params);
            $response->getBody()->write(
                json_encode([
                    "error" => false,
                    "message" => "",
                    "data" => $result,
                    "pagination" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        } catch (\Exception $e) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => $e->getMessage(),
                    "data" => [],
                ])
            );

            return $response->withHeader("Content-Type", "application/json");
        }

        return $result;
    }

    public function getProducts(Request $request, Response $response) {
        $params = $request->getQueryParams();
        $result = $this->iFoodMenuService->getProducts($params['search'], $params['page'], $params['limit']);

        if (empty($result)) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => "Nenhum item foi encontrado.",
                    "data" => [],
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );
        } else {

        $response->getBody()->write(
            json_encode([
                "error" => false,
                "message" => "",
                "data" => $result[0],
                "pagination" => [
                    "limit" => $params["limit"] ?? 10,
                    "page" => $params["page"] ?? 1,
                    "total" => $result[1][0]["TOTAL"],
                ],
            ])
        );
    }
        return $response->withHeader("Content-Type", "application/json");
    }

    public function getOccurrences(Request $request, Response $response) {
        $params = $request->getQueryParams();
        $result = $this->iFoodMenuService->getOccurrences(
            $params['search'],$params['idControlaObs'],
            $params['page'], $params['limit']
            );

        if (empty($result)) {
            $response->getBody()->write(
                json_encode([
                    "error" => true,
                    "message" => "Nenhum item foi encontrado.",
                    "data" => [],
                    "pagination" => [
                        "limit" => $params["limit"] ?? 10,
                        "page" => $params["page"] ?? 1,
                    ],
                ])
            );
        } else {
        $response->getBody()->write(
            json_encode([
                "error" => false,
                "message" => "",
                "data" => $result[0],
                "pagination" => [
                    "limit" => $params["limit"] ?? 10,
                    "page" => $params["page"] ?? 1,
                    "total" => $result[1][0]["TOTAL"],
                ],
            ])
        );
    }
        return $response->withHeader("Content-Type", "application/json");
    }

}
