<?php

namespace App\Factory;

use Doctrine\DBAL\Connection;

class iFoodMenu
{
    private $connection;

    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
    }

    public function getBranchIdToken($CDFILIAL) {
      $sql = "
        SELECT
            CDFILIAL,
            IDTOKEN,
            NVL(DSTOKENFILI, 'N/A') ||' - '|| IDTOKEN AS FILIAL
        FROM
            INTEGRACAOFILI
        WHERE
            CDINTEGRACAO = '001' AND CDFILIAL = :CDFILIAL
      ";

      $stmt = $this->connection->prepare($sql);
      $stmt->bindValue("CDFILIAL", $CDFILIAL);
      $result = $stmt->executeQuery();
      return $result->fetchAllAssociative();
    }

    public function getProducts($nmProduto, $pagina, $tamanho) {
        $sqlTotalRegistros = "
        SELECT COUNT(*) as TOTAL
        FROM
            PRODUTO
        WHERE
            LOWER(NMPRODUTO) LIKE '%' || LOWER(:NMPRODUTO) || '%'
        AND
            LENGTH(CDPRODUTO) >= 10
        ";

        $stmtTotalRegistros = $this->connection->prepare($sqlTotalRegistros);
        $stmtTotalRegistros->bindValue("NMPRODUTO", $nmProduto);;
        $totalRegistros = $stmtTotalRegistros->executeQuery();
        $totalRegistros = $totalRegistros->fetchAllAssociative();

        $sql = "
        SELECT
            NMPRODUTO,
            CDPRODUTO
        FROM
            PRODUTO
        WHERE
            LOWER(NMPRODUTO) LIKE '%' || LOWER(:NMPRODUTO) || '%'
        AND
            LENGTH(CDPRODUTO) >= 10
        OFFSET (:PAGE-1) * :LIMIT ROWS
        FETCH NEXT :LIMIT ROWS ONLY
        ";
        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("NMPRODUTO", $nmProduto);
        $stmt->bindValue("PAGE", $pagina);
        $stmt->bindValue("LIMIT", $tamanho);
        $result = $stmt->executeQuery();
        $result = $result->fetchAllAssociative();

        return [$result, $totalRegistros];
    }

    public function getOccurrences($search, $idControlaObs, $pagina, $tamanho) {
        $sqlTotal =  "
        SELECT COUNT(*) as TOTAL
        FROM
            OCORRENCIA
            WHERE
            LOWER(DSOCORR) LIKE '%' || LOWER(:DSOCORR) || '%'
        AND (
            (IDCONTROLAOBS = 'A' AND :IDCONTROLAOBS = 'A')
            OR
            IDCONTROLAOBS IN ('R', 'O') AND :IDCONTROLAOBS <> 'A'
        )
        ";
        $stmtTotal= $this->connection->prepare($sqlTotal);
        $stmtTotal->bindValue("DSOCORR", $search);
        $stmtTotal->bindValue("IDCONTROLAOBS", $idControlaObs);
        $total = $stmtTotal->executeQuery();
        $total= $total->fetchAllAssociative();

        $sql = "
        SELECT
            DSOCORR AS NMPRODUTO,
            CDGRPOCOR || '|' || CDOCORR AS CDPRODUTO,
            IDCONTROLAOBS
        FROM
            OCORRENCIA
        WHERE
            LOWER(DSOCORR) LIKE '%' || LOWER(:DSOCORR) || '%'
        AND (
            (IDCONTROLAOBS = 'A' AND :IDCONTROLAOBS = 'A')
            OR
            IDCONTROLAOBS IN ('R', 'O') AND :IDCONTROLAOBS <> 'A'
        )
        ORDER BY
            DSOCORR
        OFFSET (:PAGE - 1) * :LIMIT ROWS
        FETCH NEXT :LIMIT ROWS ONLY
        ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("DSOCORR", $search);
        $stmt->bindValue("IDCONTROLAOBS", $idControlaObs);
        $stmt->bindValue("PAGE", $pagina);
        $stmt->bindValue("LIMIT", $tamanho);
        $result = $stmt->executeQuery();
        $result = $result->fetchAllAssociative();

        return [$result, $total];
    }

    public function checkExternalCodeExists(&$params) {
        $sql = "
            SELECT CDPRODUTO
                FROM PRODUTO
            WHERE
                CDPRODUTO = :CDPRODUTO
            AND
                CDPRODUTO >= 10
        ";
        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("CDPRODUTO", $params['externalCode']);
        $result = $stmt->executeQuery();
        $result = $result->fetchAllAssociative();
        if (empty($result)) {
            throw new \Exception(
                "Não existe nenhum item correspondente ao código inserido. ".
                "Verifique o código digitado e tente novamente"
            );
        } else {
            return $result[0]['CDPRODUTO'];
        }
    }

    public function checkOcurrenceExists($params) {
        $sql = "
        SELECT CDPRODUTO
            FROM OCORRENCIA
        WHERE
            CDGRPOCOR = :CDGRPOCOR
        AND
            CDOCORR = :CDOCORR
        AND (
            IDCONTROLAOBS = :IDCONTROLAOBS
        OR
            IDCONTROLAOBS IN ('R', 'O')
        )
        ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("CDGRPOCOR", $params['cdgrpocor']);
        $stmt->bindValue("CDOCORR", $params['cdocorr']);
        $stmt->bindValue("IDCONTROLAOBS", $params['idControlaObs']);
        $result = $stmt->executeQuery();
        $result = $result->fetchAllAssociative();

        if (count($result) === 0) {
            throw new \Exception(
                "Não existe uma ocorrencia associada ao codigo informado. ".
                "Verifique o código digitado e tente novamente"
            );
        } elseif($result[0]['CDPRODUTO'] === null) {
            throw new \Exception("Ocorrência não possui um código de produto");
        } else {
            return $result[0]['CDPRODUTO'];
        }
    }

    public function getBranch() {
        $sql = "
            SELECT DISTINCT
                I.CDFILIAL,
                I.CDFILIAL || ' - ' || F.NMFILIAL AS FILIAL
            FROM
                INTEGRACAOFILI I
            INNER JOIN
                FILIAL F ON I.CDFILIAL = F.CDFILIAL
            WHERE
                I.CDINTEGRACAO = '001'
            ORDER BY
                I.CDFILIAL
            ";

        $stmt = $this->connection->prepare($sql);
        $result = $stmt->executeQuery();
        return $result->fetchAllAssociative();
    }

    public function getMerchantId($CDINTEGRACAO) {
        $sql = "
            SELECT
                CDINTEGRACAO,
                CDFILIAL,
                IDTOKEN
            FROM
                INTEGRACAOFILI
            WHERE
                (INSTR(:CDINTEGRACAO, CDINTEGRACAO) > 0 OR 'T' = :CDINTEGRACAO)
            ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("CDINTEGRACAO", $CDINTEGRACAO);
        $result = $stmt->executeQuery();
        return $result->fetchAllAssociative();
    }

    public function logItemAlteration($params)
    {
        try {
            $sql = "
            INSERT INTO LOGALTPRODCARD
                (CDINTEGRACAO, CDFILIAL, IDEXTCARD, CDOPERADOR, CDPRODUTO,
                IDEXTPROD, NMPRODCARD, DSALTERA, CDEXTANT, CDEXTATU, VRPRECANT, VRPRECATU, TPALTERA)

             VALUES
                (:CDINTEGRACAO,:CDFILIAL,:IDEXTCARD,:CDOPERADOR,:CDPRODUTO,
               :IDEXTPROD, :NMPRODCARD, :DSALTERA,:CDEXTANT,:CDEXTATU,:VRPRECANT,:VRPRECATU,:TPALTERA)
            ";
            $stmt = $this->connection->prepare($sql);
            $stmt->bindValue("CDINTEGRACAO", $params['cdIntegracao']);
            $stmt->bindValue("CDFILIAL", $params['cdFilial']);
            $stmt->bindValue("IDEXTCARD", $params['menuId']);
            $stmt->bindValue("CDOPERADOR", $params['cdOperador']);
            $stmt->bindValue("CDPRODUTO", $params['cdProduto']);
            $stmt->bindValue("IDEXTPROD", $params['productId']);
            $stmt->bindValue("NMPRODCARD", $params['nmProduto']);
            $stmt->bindValue("DSALTERA", $params['dsAltera']);
            $stmt->bindValue("CDEXTANT", $params['previousExternalCode']);
            $stmt->bindValue("CDEXTATU", $params['externalCode']);
            $stmt->bindValue("VRPRECANT", $params['previousPrice']);
            $stmt->bindValue("VRPRECATU", $params['productPrice']);
            $stmt->bindValue("TPALTERA", $params['modificationType']);
            $stmt->executeQuery();
        } catch (\Exception $e) {
            throw new \Exception("Não foi possível salvar as alterações feitas no item: ".$e);
        }
    }

    public function getAlterationLogs($params) {
        $totalRows = "
        SELECT COUNT(*) as TOTAL
        FROM
            LOGALTPRODCARD
            WHERE
            CDINTEGRACAO = :CDINTEGRACAO
        AND
            IDEXTCARD = :IDEXTCARD
        ";

        $totalRows = $this->connection->prepare($totalRows);
        $totalRows->bindValue("IDEXTCARD", $params['menuID']);
        $totalRows->bindValue("CDINTEGRACAO", $params['cdIntegracao']);
        $totalRows = $totalRows->executeQuery();
        $totalRows = $totalRows->fetchAllAssociative();

        $sql = "
        SELECT
            L.CDLOG,
            TO_CHAR(L.DTINCLUSAO, 'DD/MM/YYYY HH24:MI:SS') AS DTINCLUSAO,
            L.DSALTERA,
            L.NMPRODCARD,
            O.NMOPERADOR || ' ('||L.CDOPERADOR||')' AS AUTOR,
            CASE WHEN L.VRPRECANT <> L.VRPRECATU THEN L.VRPRECANT ELSE NULL END AS ANTERIOR_VRPREC,
            CASE WHEN L.CDEXTANT <> L.CDEXTATU THEN L.CDEXTANT ELSE NULL END AS ANTERIOR_CDEXT,
            CASE WHEN L.VRPRECANT <> L.VRPRECATU THEN L.VRPRECATU ELSE NULL END AS ATUAL_VRPREC,
            CASE WHEN L.CDEXTANT <> L.CDEXTATU THEN L.CDEXTATU ELSE NULL END AS ATUAL_CDEXT
        FROM
            LOGALTPRODCARD L
        INNER JOIN
            OPERADOR O ON L.CDOPERADOR = O.CDOPERADOR
        WHERE
            L.CDINTEGRACAO = :CDINTEGRACAO
        AND
            L.IDEXTCARD = :IDEXTCARD
        AND (
            :SEARCH IS NULL
            OR (
                LOWER(L.DSALTERA) LIKE LOWER('%'||:SEARCH||'%')
                OR LOWER(O.NMOPERADOR) LIKE LOWER('%'||:SEARCH||'%')
                OR LOWER(TO_CHAR(L.DTINCLUSAO, 'DD/MM/YYYY HH24:MI:SS')) LIKE LOWER('%'||:SEARCH||'%')
            )
        )
        ORDER BY
            L.DTINCLUSAO DESC
        OFFSET (:PAGE-1) * :LIMIT ROWS
        FETCH NEXT :LIMIT ROWS ONLY
        ";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindValue("IDEXTCARD", $params['menuID']);
        $stmt->bindValue("CDINTEGRACAO", $params['cdIntegracao']);
        $stmt->bindValue("PAGE", $params['page']);
        $stmt->bindValue("SEARCH", $params['search'] ?? null);
        $stmt->bindValue("LIMIT", $params['limit']);
        $result = $stmt->executeQuery();
        $result = $result->fetchAllAssociative();
        return [$result, $totalRows[0]];
    }
}
