<?php
namespace App\Middlewares;

use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;
use Slim\Psr7\Response;
use App\Middlewares\JwtManager as Jwt;

class JwtMiddleware {

    private $jwtManager;

    public function __construct() {
        $this->jwtManager = new Jwt();
    }

    public function __invoke(Request $request, RequestHandler $handle) {
        $response        = $handle->handle($request);
        $existingContent = (string) $response->getBody();
        $token           = $request->getHeaderLine('Authorization');
        $requestUri      = $request->getUri()->getPath();
        $uri             = explode('/index.php', $requestUri)[1];

        $response = new Response();
        $response->withHeader('Content-Type', 'application/json');
        
        if (!$this->shouldIgnoreRoute($uri)) {
            if ($token && $this->jwtManager->verifyJwtToken($token)) {
                return $response;
            } elseif (!$token && !$this->jwtManager->verifyJwtToken($token)) {
                $response = $response->withStatus(401);
                $response->getBody()->write(json_encode([
                    'error' => true, 
                    'message' => 'Token inválido ou expirado',
                    'data' => []
                ]));

                return $response;
                }
            }
            $response->getBody()->write($existingContent);
            return $response;
        }

        public function shouldIgnoreRoute($route) {
            $ignoredRoutes = ['/operator/login'];

            return in_array($route, $ignoredRoutes);
        }
    }
    