<?php

namespace App\Service;

class iFoodMenu
{
    private $iFoodMenuFactory;

    public function __construct(\App\Factory\iFoodMenu $iFoodMenuFactory)
    {
        $this->iFoodMenuFactory = $iFoodMenuFactory;
    }

    function formatStatus($originalStatus)
    {
        return $originalStatus === "AVAILABLE" ? "Disponível" : "Indisponível";
    }

    function formatDate($originalDate)
    {
        $originalDate = date_create($originalDate);
        return date_format($originalDate, "d-m-Y H:i:s");
    }

    public function authentication()
    {
        $clientId = "b29b8feb-afff-4f49-9127-4778472ea5bc"; //dados que vão ser recebidos no futuro
        $clientSecret =
            "4x6f0n7k51u7vxjks8jazu5chtlxa733ggnj1jf8dctwodzs9au5xbt1zspi2uld9lfjc0cpf5hpvq9iapqr6ouo912oiafk682";

        $data = [
            "grantType" => "client_credentials",
            "clientId" => $clientId,
            "clientSecret" => $clientSecret,
        ];

        $postData = http_build_query($data);

        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL => "https://merchant-api.ifood.com.br/authentication/v1.0/oauth/token",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => $postData,
            CURLOPT_HTTPHEADER => [
                "Content-Type: application/x-www-form-urlencoded",
            ],
        ]);

        //linha para desativar a verificação do certificado SSL, retirar futuramente
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($curl);

        if (curl_errno($curl)) {
            echo "Erro cURL: " . curl_error($curl);
        }

        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        if ($httpCode != 200) {
            echo "Erro HTTP: " . $httpCode;
        }

        curl_close($curl);

        $responseData = json_decode($response, true);
        return $responseData["accessToken"];
    }

    public function getMenuId($merchantId, $accessToken = null)
    {
        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL =>
                "https://merchant-api.ifood.com.br/catalog/v2.0/merchants/" . $merchantId . "/catalogs",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => ["Authorization: Bearer " . $accessToken],
        ]);
        //linha para desativar a verificação do certificado SSL, retirar futuramente
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($curl);

        $decodedResponse = json_decode($response, true);

        if (curl_errno($curl)) {
            throw new \Exception("Houve um erro ao tentar obter informações deste restaurante");
        }

        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        if ($httpCode != 200) {
            throw new \Exception("Houve um erro ao tentar obter informações deste restaurante");
        }

        if ($decodedResponse === null) {
            throw new \Exception("Houve um erro ao tentar obter informações deste restaurante");
        }

        foreach ($decodedResponse as &$item) {
            $item["status"] = $this->formatStatus($item["status"]);
            $item["modifiedAt"] = $this->formatDate($item["modifiedAt"]);
        }

        curl_close($curl);

        return $decodedResponse;
    }

    public function getMerchantInfo($merchantId, $accessToken = null)
    {
        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL =>
                "https://merchant-api.ifood.com.br/merchant/v1.0/merchants/" . $merchantId,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => ["Authorization: Bearer " . $accessToken],
        ]);

        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($curl);

        if (curl_errno($curl)) {
            echo "Erro cURL: " . curl_error($curl);
        }

        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        if ($httpCode != 200) {
            echo "Erro HTTP: " . $httpCode;
        }

        $decodedResponse = json_decode($response, true);

        if ($decodedResponse === null) {
            echo "Erro ao decodificar a string JSON.";
        }

        curl_close($curl);

        return $decodedResponse;
    }

    public function getMenu($menuId, $merchantid, $accessToken = null)
    {
        $url = "https://merchant-api.ifood.com.br/catalog/v2.0/merchants/" . $merchantid . "/catalogs/" . $menuId . "/categories";

        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => ["Authorization: Bearer " . $accessToken],
        ]);

        //linha para desativar a verificação do certificado SSL, retirar futuramente
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($curl);

        if (curl_errno($curl)) {
            echo "Erro cURL: " . curl_error($curl);
        }

        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        if ($httpCode != 200) {
            throw new \Exception("Não foi possivel localizar informações deste menu");
        }

        curl_close($curl);

        $response = stripslashes($response);

        $jsonArray = json_decode($response, true);

        if ($jsonArray !== null) {
            return $jsonArray;
        } else {
            echo "Erro ao decodificar a string JSON.";
        }
    }

    public function GetCategory($menu)
    {
        $result = [];
        foreach ($menu as $category) {
            $categoryInfo = [
                "name"     => $category["name"],
                "id"       => $category["id"],
                "status"   => $this->formatStatus($category["status"]),
                "sequence" => $category["sequence"],
                "index"    => $category["index"],
                "template" => $category["template"],
            ];

            $result[] = $categoryInfo;
        }
        return $result;
    }

    private function requestIFoodItems($params) {
        $search = isset($params["search"]) ? $params["search"] : "";
        $menuId = isset($params["menuID"]) ? strval($params["menuID"]) : "";
        $merchantId = isset($params["merchantId"]) ? strval($params["merchantId"]) : "";

        $token = $this->authentication();
        $menu = $this->getMenu($menuId, $merchantId, $token);

        return array_filter($menu, function ($item) use ($search) {
            return $item["id"] === $search;
        });
    }

    public function items($params) {
        $getIFoodItems = $this->requestIFoodItems($params);
        $result = array();
        if (!empty($getIFoodItems)) {
            foreach ($getIFoodItems as $category) {
                if (isset($category["items"])) {
                    foreach ($category["items"] as &$items) {
                        $this->verifyItemType($items);
                        $this->processOptions($items);
                        $result[] = $items;
                    }
                }
            }
            $result = $this->buildTree($result);
            return $result;
        }
    }

    private function verifyItemType(&$item) {
        $externalCode = $item['externalCode']?? null;
        $price = $item["price"]["value"] ?? null;

        if(!$item['hasOptionGroups']) {
            $item['itemType'] = 'simple';
        } else {
            if (empty($externalCode) && empty($price)) {
                $item['itemType'] = 'combo';
            } elseif (strlen($externalCode) === 10 && $price >= 0.01) {
                $item['itemType'] = 'withAddons';
            } else {
                $item['itemType'] = 'invalid';
            }
        }
    }

    private function processOptions(&$item) {
        if(isset($item['itemType']) && $item['itemType'] === 'simple') {
            $this->checkIsValidItemSimple($item);
            return;
        }

        foreach ($item['optionGroups'] as &$optionGroup) {
            $optionGroup['parent_id'] = $item['id'];
            $optionGroup['item_id'] = $item['id'];
            foreach ($optionGroup['options'] as &$option) {
                $option['parent_id'] = $optionGroup['id'];
                $option['item_id'] = $item['id'];
                $this->checkIsValidOption($item, $option);
            }
        }
    }

    private function checkIsValidItemSimple(&$item) {
        $price = $item['price']['value'] ?? null;
        $externalCode = $item['externalCode'] ?? null;
        $item['isValid'] = $price >= 0.01 && strlen($externalCode) === 10;
    }

    private function checkIsValidOption(&$item, &$option) {
        $externalCode = $option["externalCode"] ?? null;
        $price = $option["price"]["value"] ?? null;
        $itemType = $item['itemType'];

        switch($itemType) {
            case 'combo':
                $option['isValid'] = $price >= 0.01 && strlen($externalCode) === 10;
                break;
            case 'withAddons':
                $option['isValid'] = (empty($price) && strstr($externalCode, '|')) ||
                                     ($price >= 0.01 && strlen($externalCode) === 10 || strstr($externalCode, '|'));
                break;
            default:
                $option['isValid'] = false;
                break;
        }

        if(!$option['isValid']) {
            $item['hasInvalidOptions'] = true;
        }
    }

    private function buildTree($items) {
        if(!empty($items)) {
            foreach ($items as $item) {
                $isValid = isset($item['hasInvalidOptions']) ? !$item['hasInvalidOptions'] : true;
                $result[] = [
                                "id"                => $item['id'],
                                "name"              => $item['name'],
                                "externalCode"      => $item['externalCode'] ?? null,
                                "price"             => $item['price']['value'],
                                "itemType"          => $item['itemType'],
                                "isValid"           => $isValid,
                            ];
                if(isset($item['optionGroups'])) {
                    foreach($item['optionGroups'] as $group) {
                        $result[] = [
                                        "id"        => $group["id"],
                                        "parent_id" => $group['parent_id'],
                                        "item_id"   => $group['item_id'],
                                        "name"      => $group["name"],
                                        "isValid"   => $isValid,
                                    ];

                        if(isset($group['options'])) {
                            foreach($group['options'] as $option){
                                $result[] = [
                                    "id"           => $option['id'],
                                    "price"        => $option['price']['value'],
                                    "parent_id"    => $option['parent_id'],
                                    "item_id"      => $option['item_id'],
                                    "name"         => $option['name'],
                                    "externalCode" => $option['externalCode'] ?? null,
                                    "isValid"      => $isValid,
                                ];
                            }
                        }
                    }
                }
            }
        }
        return $result;
    }

    public function setOptionExternalCode($accessToken, $params) {
        $merchantId = $params['merchant'];

        $data = [
            "optionId"                    => $params['productId'],
            "externalCode"                => $params['externalCode'],
            "parentCustomizationOptionId" => null,
            "externalCodeByCatalog" => [
                [
                  "externalCode"   => $params['externalCode'],
                  "catalogContext" => $params['menuId']
                ]
              ]
        ];
        $postData = http_build_query($data);

        $data = json_encode($data);

        $url = "https://merchant-api.ifood.com.br/catalog/v2.0/merchants/" . $merchantId . "/options/externalCode";

        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "PATCH",
            CURLOPT_POSTFIELDS => $postData,
            CURLOPT_HTTPHEADER => ["Authorization: Bearer " . $accessToken],
        ]);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        $curlResponse = curl_exec($curl);

        if (curl_errno($curl)) {
            throw new \Exception("Ocorreu um erro ao tentar atualizar esta opção.");
        }

        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if ($httpCode != 200) {
            throw new \Exception("Ocorreu um erro ao tentar atualizar esta opção.");
        }

        curl_close($curl);

        return [];
    }

    public function setItemExternalCode($accessToken, $params) {
        $merchantId = $params['merchant'];
        $data = [
            "itemId"                => $params['productId'],
            "externalCode"          => $params['externalCode'],
            "externalCodeByCatalog" => [
                [
                  "externalCode"   => $params['externalCode'],
                  "catalogContext" => $params['menuId']
                ]
              ]
        ];

        $postData = http_build_query($data);

        $data = json_encode($data);

        $url = "https://merchant-api.ifood.com.br/catalog/v2.0/merchants/".$merchantId."/items/externalCode";

        $curl = curl_init();

        curl_setopt_array($curl, [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "PATCH",
            CURLOPT_POSTFIELDS => $postData,
            CURLOPT_HTTPHEADER => ["Authorization: Bearer " . $accessToken],
        ]);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

        $curlResponse = curl_exec($curl);

        if (curl_errno($curl)) {
            throw new \Exception("Não foi possivel atualizar o item do cardápio.");
        }

        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        if ($httpCode != 200) {
            throw new \Exception("Não foi possivel atualizar o item do cardápio.");
        }

        curl_close($curl);



        return [];
    }

    public function getBranch() {
        return $this->iFoodMenuFactory->getBranch();
    }

    function formatPrice($price) {

        $formattedPrice = 'R$'.number_format($price, 2, ',', '.');

        return $formattedPrice;
    }

    public function describeAlteration(&$params) {
        $description = '';

        if ($params['previousPrice'] != $params['productPrice']) {
            $description = 'Preço alterado de '.$this->formatPrice($params['previousPrice']).
                            ' para '.$this->formatPrice($params['productPrice']);
        }

        if ($params['previousExternalCode'] != $params['externalCode']) {
            $description .= 'Código externo alterado de '.$params['previousExternalCode'].
            ' para '.$params['externalCode'].'';
        }
        if($params['modificationType'] === 'I'){
            $description .= ' no item "'.$params['nmProduto'].'" ';

        } else {
            $description .= ' no complemento "'.$params['nmProduto'].'" ';
        }
        $params['dsAltera'] = $description;
    }

    public function getBranchIdToken($cdFilial) {
        return $this->iFoodMenuFactory->getBranchIdToken($cdFilial);
    }

    public function checkExternalCodeExists($cdProduto) {
       return $this->iFoodMenuFactory->checkExternalCodeExists($cdProduto);
    }

    public function checkOcurrenceExists($params) {
       return $this->iFoodMenuFactory->checkOcurrenceExists($params);
    }

    public function getProducts($nmProduto, $page, $limit) {
        return $this->iFoodMenuFactory->getProducts($nmProduto, $page, $limit);
    }

    public function getOccurrences($search, $idControlaObs, $page, $limit) {
        return $this->iFoodMenuFactory->getOccurrences($search, $idControlaObs, $page, $limit);
    }

    public function logItemAlteration(&$params) {
       $this->describeAlteration($params);
       return $this->iFoodMenuFactory->logItemAlteration($params);
    }

    public function getAlterationLogs($params) {
        $result = [];
        $data = $this->iFoodMenuFactory->getAlterationLogs($params);

        foreach ($data[0] as &$row) {
            $result = array_merge($result, $this->formattedLogResult($row));
        }

        $totalRows = $data[1]['TOTAL'];
        return [$result, $totalRows];
    }

    private function formattedLogResult($row) {

        if ($row['ANTERIOR_VRPREC'] != $row['ATUAL_VRPREC'] &&
        $row['ANTERIOR_CDEXT'] != $row['ATUAL_CDEXT']) {
        $result = [];

        $result[] = [
            'CDLOG'      => $row['CDLOG'],
            'DTINCLUSAO' => $row['DTINCLUSAO'],
            'AUTOR'      => $row['AUTOR'],
            'DSALTERA'   => $row['NMPRODCARD'],
            'ANTERIOR'   => null,
            'ATUAL'      => null,
        ];
        $result[] = [
            'CDLOG'      => $row['CDLOG'].'(1)',
            'DTINCLUSAO' => null,
            'PARENT_ID'  => $row['CDLOG'],
            'AUTOR'      => null,
            'DSALTERA'   => 'Preço Alterado.',
            'ANTERIOR'   => $this->formatPrice($row['ANTERIOR_VRPREC']),
            'ATUAL'      => $this->formatPrice($row['ATUAL_VRPREC']),
        ];
        $result[] = [
            'CDLOG'      => $row['CDLOG'].'(2)',
            'DTINCLUSAO' => null,
            'PARENT_ID'  => $row['CDLOG'],
            'AUTOR'      => null,
            'DSALTERA'   => 'Código Externo Alterado.',
            'ANTERIOR'   => $row['ANTERIOR_CDEXT'],
            'ATUAL'      => $row['ATUAL_CDEXT'],
        ];
    } else {
        $previousValue = '';
        $newValue = '';

        if(!empty($row['ANTERIOR_VRPREC']) && empty(!$row['ATUAL_VRPREC'])){
            $previousValue = $this->formatPrice($row['ANTERIOR_VRPREC']);
            $newValue =   $this->formatPrice($row['ATUAL_VRPREC']);
        }

        if(!empty($row['ANTERIOR_CDEXT']) && empty(!$row['ATUAL_CDEXT'])){
            $previousValue = $row['ANTERIOR_CDEXT'];
            $newValue =   $row['ATUAL_CDEXT'];
        }

        $result[] = [
            'CDLOG'      => $row['CDLOG'],
            'DTINCLUSAO' => $row['DTINCLUSAO'],
            'AUTOR'      => $row['AUTOR'],
            'DSALTERA'   => $row['DSALTERA'],
            'ANTERIOR'   => $previousValue,
            'ATUAL'      => $newValue,
        ];
        }
        return $result;
    }
}
